// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_bar.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
static int _bar_io_acquire(os_bar_t* bar)
{
	int	ret;

	for (;;)	// A convenience loop.
	{
		if ((bar->flags & D0) == 0)
		{
			// The region is not I/O mapped.
			ret	= -EINVAL;
			db_printf("%s: BAR%d not I/O mapped.\n", DEV_NAME, (int) bar->index);
			break;
		}

		// INtime does not include a mechanism for associating a region
		// with a single device or a driver.
		ret				= 0;
		bar->requested	= 1;
		bar->vaddr		= (VADDR_T) bar->phys_adrs;
		break;
	}

	return(ret);
}




//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
static int _bar_mem_acquire(os_bar_t* bar)
{
	int		ret;
	void*	vp;

	for (;;)	// A convenience loop.
	{
		if (bar->flags & D0)
		{
			// The region is not memory mapped.
			ret	= -EINVAL;
			db_printf("%s: BAR%d not memory mapped.\n", DEV_NAME, (int) bar->index);
			break;
		}

		// INtime does not include a mechanism for associating a region
		// with a single device or a driver.
		bar->requested	= 1;
		vp				= MapRtPhysicalMemory(bar->phys_adrs, bar->size);

		if (vp == NULL)
		{
			// This BAR region request failed.
			ret	= os_get_error_status();
			db_printf(	"%s: Failed to map BAR%d to driver virtual address space.\n",
						DEV_NAME,
						(int) bar->index);
			break;
		}

		// All went well.
		ret			= 0;
		bar->vaddr	= (VADDR_T) vp;
		break;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_bar_create(dev_data_t* dev, int index, os_bar_t* bar)
{
	int	ret;

	for (;;)	// A convenience loop.
	{
		if ((dev == NULL) || (bar == NULL))
		{
			// This should never happen.
			ret	= -EINVAL;
			break;
		}

		memset(bar, 0,sizeof(os_bar_t));
		bar->dev		= dev;
		bar->index		= index;
		bar->offset		= 0x10 + 4 * index;
		bar->phys_adrs	= dev->pci.pcidev.IoSpace[index].start;
		bar->size		= dev->pci.pcidev.IoSpace[index].size;
		bar->reg		= os_reg_pci_rx_u32(dev, 1, bar->offset);

		if (bar->reg == 0)
		{
			bar->flags		= 0;
			bar->io_mapped	= 0;
			bar->rw			= 0;
			ret				= 0;
		}
		else if (bar->reg & D0)
		{
			bar->flags		= bar->reg & 0x3;
			bar->io_mapped	= 1;
			bar->rw			= 1;
			ret				= _bar_io_acquire(bar);
		}
		else
		{
			bar->flags		= bar->reg & 0xF;
			bar->io_mapped	= 0;
			bar->rw			= 1;
			ret				= _bar_mem_acquire(bar);
		}

		if (ret)
			db_printf("%s: Failed to map BAR%d.\n", DEV_NAME, (int) bar->index);

		break;
	}

	return(ret);
}



//*****************************************************************************
void os_bar_destroy(os_bar_t* bar)
{
	for (;;)	// A convenience loop.
	{
		if (bar == NULL)
		{
			// This should never happen.
			break;
		}

		if (bar->requested == 0)
		{
			// There is nothing to release.
		}
		else if (bar->io_mapped)
		{
			// INtime does not include a mechanism for associating a region
			// with a single device or a driver.
			bar->requested	= 0;
			bar->vaddr		= 0;
		}
		else
		{
			// INtime does not include a mechanism for associating a region
			// with a single device or a driver.

			if (bar->vaddr)
				FreeRtMemory((void*) bar->vaddr);	// Assume it works.

			bar->requested	= 0;
			bar->vaddr		= 0;
		}

		memset(bar, 0,sizeof(os_bar_t));
		break;
	}
}


