// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_main.h $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: header file

#ifndef __OS_MAIN_H__
#define __OS_MAIN_H__

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rt.h"
#include "pcibus.h"



// #defines *******************************************************************

// This is for the OS specific code.
#define OS_COMMON_VERSION					"4"
// 4	Updated to INtime 6.3.
// 3	Corrected an interrupt acquisition bug.
// 2	LINTed sources.
// 1	Made a correction to the code for exiting the driver.
//		Added definition of ERESTARTSYS.
//		Updated the code for acquiring and releasing the interrupt.
//		Updated the PCI Enable/Disable code.
//		Improved some error reporting.
//		Updated semaphore processing code.
//		Removed the -u command line option.
// 0	Initial release.

// Printing is unsupported in ISR.
#undef	PRINTF_ISR

#define	INTIME_ERROR_OFFSET					1000

#define	INTIME_MAX_DEV_LIST_SIZE			32

#define os_mem_copy_from_user_ioctl			os_mem_copy_from_user
#define os_mem_copy_from_user_tx			os_mem_copy_from_user
#define os_mem_copy_to_user_ioctl			os_mem_copy_to_user
#define os_mem_copy_to_user_rx				os_mem_copy_to_user



// data types *****************************************************************

typedef QWORD				os_time_tick_t;
typedef	long				VADDR_T;

typedef struct					// Don't forget: _pci_region in gsc_reg.c
{
	// These fields MUST be kept as is.
	int					index;		// BARx
	int					offset;		// Offset of BARx register in PCI space.
	u32					reg;		// Actual BARx register value.
	u32					flags;		// lower register bits
	int					io_mapped;	// Is this an I/O mapped region?
	unsigned long		phys_adrs;	// Physical address of region.
	u32					size;		// Region size in bytes.
	int					rw;			// Read/Write access? If not, then read-only.

	// The following are not order specific.

	// These are computed when mapped in.
	u32					requested;	// Is resource requested from OS?
	VADDR_T				vaddr;		// Kernel virtual address.

	// Any OS specific fields must go after this point.
	dev_data_t*			dev;
} os_bar_t;


typedef struct
{
	int	unused;
} os_data_t;

typedef struct				// This is for the Driver Mailbox
{
	int						started;			// indicates if started working
	int						stopped;			// indicates if stopped working
	int						exit_msg;			// indicated if EXIT MESSAGE was received
	int						failed;				// indicates line # if startup failed
	RTHANDLE				mbox;				// Mailbox for DEV, not DEV.X
	char					buffer[8 * 1024];	// Must hold global semaphore to fill buffer.
} os_driver_t;

typedef struct
{
	KNHANDLE				handle;
	u8						sem_data[KN_SEMAPHORE_SIZE];
} os_event_t;

typedef	struct
{
	RTHANDLE				sem;
} os_sem_t;

typedef struct
{
	int						started;			// indicates if thread started working
	int						stopped;			// indicates if loop has stopped
	int						ret;				// Reports error status
	int						failed;				// indicates line # if startup failed
	int						installed;			// indicates if ISR was installed
	int						do_exit;			// If exit, then termiate processing
	int						told_to_exit;		// indicated if EXIT MESSAGE was received
	WORD					irq_level;			// initial then encoded
	os_sem_t				sem;				// For startup sequencing
	RTHANDLE				thread;				// Handle for thread
} os_irq_t;

typedef struct
{
	unsigned long			adrs;
	size_t					bytes;
	void*					alloc_ptr;	// DO NOT USE THIS FOR DMA!
	void*					ptr;
} os_mem_t;

typedef struct
{
	PCIDEV					pcidev;
	int						enabled;
} os_pci_t;

typedef struct
{
	RTHANDLE				mbox;
	GSC_ALT_STRUCT_T*		alt;		// Used for shutdown.
	u8						initialized;
	u8						busy;
} os_service_data_t;		// Used by each thread that processes main messages.

typedef struct
{
	os_sem_t				sem;
	os_service_data_t		data[100];
} os_service_threads_t;		// Tracks service threads.

typedef struct
{
	int						in_use;
	int						ret;		// Tell the main thread there was a problem.
	int						started;	// Tell the main thread we started.
	int						exit;		// Wait til we're told we can exit.
	GSC_ALT_STRUCT_T*		alt;
	int						index;
	RTHANDLE				mbox;

} os_dev_thread_t;			// Passes essential data to main device threads.

// INtime has no spinlocks.
// Since these aren't used inside the ISR we use spinlocks.
typedef os_sem_t			os_spinlock_t;

typedef struct
{
	QWORD					count;	// low level ticks
} os_time_t;

typedef struct
{
	int						dev_qty;
	int						priority;		// for all threads
	int						driver_loaded;
	long					tick_rate;
	long					tick_us;
	int						verbose;

	os_driver_t				driver;
	os_dev_thread_t			device_threads[INTIME_MAX_DEV_LIST_SIZE];
	os_service_threads_t	service_threads;

	// The list must be locked for opening and closing.
	os_sem_t				sem;
	dev_data_t*				dev_list[INTIME_MAX_DEV_LIST_SIZE];
} gsc_global_t;



// prototypes *****************************************************************

int				db_printf(const char *format, ...);

int				os_bar_create(dev_data_t* dev, int index, os_bar_t* bar);
void			os_bar_destroy(os_bar_t* bar);

int				os_close(GSC_ALT_STRUCT_T* alt);

void			os_event_create(os_event_t* evnt);
void			os_event_destroy(os_event_t* evnt);
void			os_event_resume(os_event_t* evnt);
void			os_event_wait(os_event_t* evnt, os_time_tick_t timeout);
void			os_exit_driver(void);
int				os_exit_initiate(const char* prefix);

int				os_get_error_status(void);
int				os_get_root_process(RTHANDLE* root);

int				os_init_driver(void);
int				os_ioctl(GSC_ALT_STRUCT_T* alt, long cmd, void* arg);
int				os_ioctl_init(void);
void			os_ioctl_reset(void);
int				os_irq_acquire(dev_data_t* dev);
int				os_irq_create(dev_data_t* dev);
void			os_irq_destroy(dev_data_t* dev);
void			os_irq_release(dev_data_t* dev);

int				os_mbox_devices_start(void);
int				os_mbox_driver_exists(void);
void			os_mbox_driver_start(void);
int				os_mem_copy_from_user(void* dst, const void* src, long size);
int				os_mem_copy_to_user(void* dst, const void* src, long size);
void*			os_mem_data_alloc(size_t size);
void			os_mem_data_free(void* ptr);
void*			os_mem_dma_alloc(size_t* size, os_mem_t* mem);
void			os_mem_dma_free(os_mem_t* mem);

int				os_open(GSC_ALT_STRUCT_T* alt, int share);

void			os_pci_dev_disable(os_pci_t* pci);
int				os_pci_dev_enable(os_pci_t* pci);
void			os_pci_master_clear(os_pci_t* pci);
int				os_pci_master_set(os_pci_t* pci);

long			os_read(GSC_ALT_STRUCT_T* alt, void* buf, size_t count);
u8				os_reg_mem_mx_u8	(dev_data_t* dev, VADDR_T va, u8  value, u8  mask);
u16				os_reg_mem_mx_u16	(dev_data_t* dev, VADDR_T va, u16 value, u16 mask);
u32				os_reg_mem_mx_u32	(dev_data_t* dev, VADDR_T va, u32 value, u32 mask);
u8				os_reg_mem_rx_u8	(dev_data_t* dev, VADDR_T va);
u16				os_reg_mem_rx_u16	(dev_data_t* dev, VADDR_T va);
u32				os_reg_mem_rx_u32	(dev_data_t* dev, VADDR_T va);
void			os_reg_mem_tx_u8	(dev_data_t* dev, VADDR_T va, u8 value);
void			os_reg_mem_tx_u16	(dev_data_t* dev, VADDR_T va, u16 value);
void			os_reg_mem_tx_u32	(dev_data_t* dev, VADDR_T va, u32 value);
u8				os_reg_pci_mx_u8	(dev_data_t* dev, int lock, u16 offset, u8  value, u8  mask);
u16				os_reg_pci_mx_u16	(dev_data_t* dev, int lock, u16 offset, u16 value, u16 mask);
u32				os_reg_pci_mx_u32	(dev_data_t* dev, int lock, u16 offset, u32 value, u32 mask);
u8				os_reg_pci_rx_u8	(dev_data_t* dev, int lock, u16 offset);
u16				os_reg_pci_rx_u16	(dev_data_t* dev, int lock, u16 offset);
u32				os_reg_pci_rx_u32	(dev_data_t* dev, int lock, u16 offset);
void			os_reg_pci_tx_u8	(dev_data_t* dev, int lock, u16 offset, u8 value);
void			os_reg_pci_tx_u16	(dev_data_t* dev, int lock, u16 offset, u16 value);
void			os_reg_pci_tx_u32	(dev_data_t* dev, int lock, u16 offset, u32 value);

int				os_sem_create(os_sem_t* sem);
int				os_sem_create_qty(int max_qty, int start_qty, os_sem_t* sem);
int				os_sem_destroy(os_sem_t* sem);
int				os_sem_lock(os_sem_t* sem);
int				os_sem_unlock(os_sem_t* sem);
int				os_spinlock_create(os_spinlock_t* lock);
int				os_spinlock_destroy(os_spinlock_t* lock);
int				os_spinlock_lock(os_spinlock_t* lock);
int				os_spinlock_unlock(os_spinlock_t* lock);

long			os_time_delta_ms(const os_time_t* tt1, const os_time_t* tt2);
void			os_time_get(os_time_t* tt);
os_time_tick_t	os_time_ms_to_ticks(long ms);
os_time_tick_t	os_time_tick_get(void);
int				os_time_tick_rate(void);
int				os_time_tick_sleep(int ticks);
int				os_time_tick_timedout(os_time_tick_t tick_time);
long			os_time_ticks_to_ms(os_time_tick_t ticks);
void			os_time_us_delay(long us);
void			os_time_sleep_ms(long ms);

void			os_validate(void);						// must define __OS_VALIDATE__ to use

long			os_write(GSC_ALT_STRUCT_T* alt, const void* buf, size_t count);



#endif
