// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_mem.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_mem_copy_from_user(void* dst, const void* src, long size)
{
	int	ret;

	if (dst && src)
	{
		// The memory pointers are both valid in the driver's address space.
		memcpy(dst, src, size);
		ret	= 0;
	}
	else
	{
		ret	= -EFAULT;
		db_printf("%s: Internal failure: %s, %d\n", DEV_NAME, __FILE__, __LINE__);

	}

	return(ret);
}



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_mem_copy_to_user(void* dst, const void* src, long size)
{
	int	ret;

	if (dst && src)
	{
		// The memory pointers are both valid in the driver's address space.
		memcpy(dst, src, size);
		ret	= 0;
	}
	else
	{
		ret	= -EFAULT;
		db_printf("%s: Internal failure: %s, %d\n", DEV_NAME, __FILE__, __LINE__);
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	!NULL	Success
//	NULL	Failure
void* os_mem_data_alloc(size_t size)
{
	void*	ptr;

	if (size == 0)
	{
		ptr	= NULL;
		db_printf("%s: Internal failure: %s, %d\n", DEV_NAME, __FILE__, __LINE__);
	}
	else
	{
		ptr	= AllocateRtMemory(size);

		if (ptr == NULL)
			db_printf("%s: os_mem_data_alloc(%ld) failed.\n", DEV_NAME, (long) size);
	}

	return(ptr);
}



//*****************************************************************************
void os_mem_data_free(void* ptr)
{
	if (ptr)
		FreeRtMemory(ptr);
	else
		db_printf("%s: Internal failure: %s, %d\n", DEV_NAME, __FILE__, __LINE__);
}




//*****************************************************************************
// Return:
//	!NULL	Success
//	NULL	Failure
void* os_mem_dma_alloc(size_t* bytes, os_mem_t* mem)
{
	QWORD	adrs;
	void*	dma;
	QWORD	max;
	void*	ptr;

	for (;;)	// A convenience loop.
	{
		if (bytes == NULL)
		{
			// A pointer is NULL.
			ptr	= NULL;
			db_printf("%s: os_mem_dma_alloc() failure: validation: bytes == NULL\n", DEV_NAME);
			break;
		}

		if (mem == NULL)
		{
			// A pointer is NULL.
			ptr	= NULL;
			db_printf("%s: os_mem_dma_alloc() failure: validation: mem == NULL\n", DEV_NAME);
			break;
		}

		if (bytes[0] == 0)
		{
			// No memory is being requested.
			memset(mem, 0, sizeof(mem[0]));
			ptr	= NULL;
			db_printf("%s: os_mem_dma_alloc() failure: validation: bytes[0] == 0\n", DEV_NAME);
			break;
		}

		ptr	= AllocateRtMemoryEx(bytes[0], 0L, 0x100000000L, 0);

		if (ptr == NULL)
		{
			// The allocation failed.
			memset(mem, 0, sizeof(mem[0]));
			db_printf("%s: os_mem_dma_alloc(%ld) failure: AllocateRtMemoryEx()\n", DEV_NAME, (long) bytes[0]);
			break;
		}

		adrs	= GetRtPhysicalAddress64(ptr, bytes[0], 0);

		if (adrs == BAD_PHYSICAL_ADDRESS)
		{
			// We couldn't get the physical address.
			FreeRtMemory(ptr);
			ptr	= NULL;
			memset(mem, 0, sizeof(mem[0]));
			db_printf("%s: os_mem_dma_alloc(%ld) failure: GetRtPhysicalAddress64()\n", DEV_NAME, (long) bytes[0]);
			break;
		}

		max	= 0xFFFFFFFFUL - bytes[0];

		if (adrs > max)
		{
			// We couldn't get 32-bit memory.
			FreeRtMemory(ptr);
			ptr	= NULL;
			memset(mem, 0, sizeof(mem[0]));
			db_printf("%s: os_mem_dma_alloc(%ld) failure: AllocateRtMemoryEx() return 64-bit memory\n", DEV_NAME, (long) bytes[0]);
			break;
		}

		dma	= MapRtPhysicalMemory64(adrs, bytes[0], PTE_CACHE_DISABLE);

		if (dma == NULL)
		{
			// We couldn't map the area as desired.
			FreeRtMemory(ptr);
			ptr	= NULL;
			memset(mem, 0, sizeof(mem[0]));
			db_printf("%s: os_mem_dma_alloc(%ld) failure: MapRtPhysicalMemoryEx()\n", DEV_NAME, (long) bytes[0]);
			break;
		}

		// The allocation and mapping was successful.
		mem->adrs		= (unsigned long) adrs;
		mem->alloc_ptr	= ptr;
		mem->ptr		= dma;
		mem->bytes		= bytes[0];
		break;
	}

	return(ptr);
}



//*****************************************************************************
void os_mem_dma_free(os_mem_t* mem)
{
	if (mem)
	{
		FreeRtMemory(mem->ptr);			// Release the cache-free mapping.
		FreeRtMemory(mem->alloc_ptr);	// Release the memory block.
		memset(mem, 0, sizeof(mem[0]));
	}
}


