// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/aout/aio.c $
// $Rev: 43279 $
// $Date: 2018-07-09 17:41:21 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// #defines *******************************************************************

#define SIZEOF_ARRAY(a)			(sizeof((a))/ sizeof((a)[0]))



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_loops;	// Number of times to send a block of data.
static	long	_qty;	// Number of samples to send per block.



/******************************************************************************
*
*	Function:	_init_data
*
*	Purpose:
*
*		initialize our transmit data buffer.
*
*	Arguments:
*
*		index	The channel index we're using.
*
*		rate	The data output clock rate.
*
*		seconds	Generate data for this period of time.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static void _init_data(int index, long rate, int seconds)
{
	#define PIE		(3.1415962)

	int			chans	= 1;	// We only access one channel at a time.
	u32*		dest;
	int			i;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	int			samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");

	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) rate) * chans;
	samp_tot	= ((float) rate) * chans * seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (rate <= 100) ? rate				//      10-      100 ->    10-   100
				: (rate <= 1000) ? (rate / 2)		//     101-    1,000 ->    50-   500
				: (rate <= 10000) ? (rate / 5)		//   1,001-   10,000 ->   200- 2,000
				: (rate <= 100000) ? (rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (rate / 100);						// 100,001-1,000,000 -> 1,000-10,000
	period_len	= (period_len <= 1) ? 2 : period_len;
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_loops	= 1;
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_loops	= seconds;
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_loops	= (long) (seconds * (samp_sec / period_tot));
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (long) ((samp_dat / chans) * chans);
		_loops	= (long) ((samp_tot + _qty - 1) / _qty);
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		if (index == 0)
			*dest++	= (u16) (0xFFFF - (0xFFFF * samp / (period_len - 1)));

		// 2nd channel: ramp wave
		if (index == 1)
			*dest++	= (u16) (0xFFFF * samp / (period_len - 1));

		// 3rd channel: square wave
		if (index == 2)
			*dest++	= (u16) (((0xFFFF * samp / (period_len - 1)) >= 0x8000) ? 0xFFFF : 0);

		// 4th channel: sine wave
		if (index == 3)
			*dest++	= (u32) (((long) 0x7FFF) * (1 + sin((2 * PIE / period_len) * samp)));
	}

	printf("DONE  (pattern length: %ld samples, %ld %s)\n", period_len, period, units);
}



/******************************************************************************
*
*	Function:	_io_write
*
*	Purpose:
*
*		Write the data to the device.
*
*	Arguments:
*
*		fd		This is the handle used to access the device.
*
*		errs	This is the number of errors seen so far. We don't do the
*				work if errors have been see.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _io_write(int fd, int errs)
{
	time_t		begin;
	time_t		end;
	long		loop;
	char*		ptr;
	long long	rate;
	int			ret;
	int			secs;
	int			send;
	int			sent	= 0;
	s32			status;
	long long	total	= 0;

	gsc_label("Writing Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED   (due to errors)\n");
			break;
		}

		errs	= 0;
		begin	= time(NULL);

		// Write the data to the device.

		for (loop = 0; loop < _loops; loop++)
		{
			send	= _qty * 4;
			ptr		= (char*) _data;

			for (;;)
			{
				sent	= ao4mf_write(fd, ptr, send);

				if (sent <= 0)
					break;

				total	+= sent;

				if (sent == send)
					break;

				ptr		+= sent;
				send	-= sent;
			}

			if (sent <= 0)
				break;
		}

		// Wait for the output buffer to empty.

		for (;;)
		{
			ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_AO_BUF_STATUS, &status);
			errs	+= ret ? 1 : 0;

			if ((errs) || (status == AO4MF_AO_BUF_STATUS_EMPTY))
				break;
		}

		// Compute the results.

		end		= time(NULL);

		if (sent < 0)
		{
			errs	= 1;
			printf("FAIL <---  (ao4mf_write error)\n");
		}
		else
		{
			if (sent == 0)
				printf("FAIL <---  (I/O timeout, ");
			else
				printf("PASS  (");

			gsc_label_long_comma(total / 4);
			printf(" Samples, ");

			secs	= end - begin;
			gsc_label_long_comma(secs);
			printf(" Seconds, ");

			rate	= total / 4 / secs;
			gsc_label_long_comma(rate);
			printf(" S/S)\n");
		}

		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_out
*
*	Purpose:
*
*		Ganerate Analog Output.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		seconds	Generate data for this period of time.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int aio_out(int fd, int seconds)
{
	int	errs	= 0;
	s32	index;
	s32	rate;

	gsc_label("Analog Output");
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_INDEX_CHANNEL, &index);
	errs	+= ao4mf_fsamp_report(fd, -1, 0, index, &rate);

	if (errs == 0)
	{
		printf("\n");
		gsc_label_level_inc();

		_init_data(index, rate, seconds);
		errs	+= ao4mf_ao_buf_clear(fd, -1, 1);
		errs	+= _io_write(fd, errs);

		gsc_label_level_dec();
	}

	return(errs);
}


